/**
 * Copyright (c) 2005 Blaine Buxton
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.btb.needle;

import static javax.swing.text.html.HTML.Tag.*;

import java.io.*;
import java.util.*;

import junit.framework.*;

public class HtmlProcessorTest extends TestCase {
    final static String SIMPLE_TAG_SAMPLE = "<html><body><h1><b>Alice</b></h1><h2>Cooper</h2><b><h1>Rules</h1></b></body></html>";
    final static String SIMPLE_ATTRIBUTE_SAMPLE_FIRST = "<html><body><h1 test_id=\"FIRST\"><b test_id=\"BOLD\">Alice</b></h1><h2>Cooper</h2><b><h1 test_id=\"SECOND\">Rules</h1></b></body></html>";
    final static String SIMPLE_ATTRIBUTE_SAMPLE_SECOND = "<html><body><h1><b>Alice</b></h1><h2>Remember the Coop? OK?</h2><b><h1>Rules</h1></b></body></html>";
    final static String COMPOUND_TAG_SAMPLE = "<html><body><h1><i><b>Alice</b></i></h1><h2>Cooper</h2><b><h1>Rules</h1></b></body></html>";
    final static String TABLE_COMPOUND_SAMPLE = "<html><body><table test_id=customers><tr test_id=cust_1><td test_id=last_name>George</td><td test_id=first_name>Clinton</td></tr><tr test_type=customer test_id=cust_2><td test_id=last_name>Cambria</td><td test_id=first_name>Coheed</td></tr></table></body></html>";

    public void testSimpleTag() throws Exception {
        HtmlRule rule = HtmlRule.onTags(H1);
        Reader reader = new StringReader(SIMPLE_TAG_SAMPLE);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(2, result.size());
        assertEquals("Alice", result.get(0).getText().trim());
        assertEquals("Rules", result.get(1).getText().trim());
        assertEquals("<h1><b>Alice</b></h1>", result.get(0).toString());
        assertEquals("<h1>Rules</h1>", result.get(1).toString());
    }

    public void testCompoundTag() throws Exception {
        HtmlRule rule = HtmlRule.onTags(H1, B);
        Reader reader = new StringReader(COMPOUND_TAG_SAMPLE);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(1, result.size());
        assertEquals("Alice", result.get(0).getText().trim());
        assertEquals("<b>Alice</b>", result.get(0).toString());
    }

    public void testSimpleAttribute() throws Exception {
        String[] attribute = { "test_id", "FIRST" };
        String[] attribute2 = { "test_id", "SEC.*" };

        HtmlRule rule = HtmlRule.onAttributes(attribute);
        Reader reader = new StringReader(SIMPLE_ATTRIBUTE_SAMPLE_FIRST);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(1, result.size());
        assertEquals("Alice", result.get(0).getText().trim());
        assertEquals("<h1 test_id=\"FIRST\"><b test_id=\"BOLD\">Alice</b></h1>", result.get(0).toString());

        rule = HtmlRule.onAttributes(attribute2);
        reader = new StringReader(SIMPLE_ATTRIBUTE_SAMPLE_FIRST);
        result = new HtmlProcessor().parse(reader, rule);
        assertEquals(1, result.size());
        assertEquals("Rules", result.get(0).getText().trim());
        assertEquals("<h1 test_id=\"SECOND\">Rules</h1>", result.get(0).toString());
    }

    public void testComplex() throws Exception {
        HtmlRule rule = HtmlRule.onTags(TABLE);
        rule.thenAttribute("test_id", "cust_1");
        rule.thenAttribute("test_id", "last_name");
        Reader reader = new StringReader(TABLE_COMPOUND_SAMPLE);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(1, result.size());
        assertEquals("George", result.get(0).getText().trim());
        assertEquals("<td test_id=\"last_name\">George</td>", result.get(0).toString());
    }

    public void testOrCompound() throws Exception {
        HtmlRule rule = HtmlRule.onTags(TABLE);
        rule.thenOr(AttributeRule.on("test_id", "cust_2"), AttributeRule.on("test_id", "cust_1"));
        rule.thenAttribute("test_id", "last_name");
        Reader reader = new StringReader(TABLE_COMPOUND_SAMPLE);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(2, result.size());
        assertEquals("George", result.get(0).getText().trim());
        assertEquals("<td test_id=\"last_name\">George</td>", result.get(0).toString());
        assertEquals("Cambria", result.get(1).getText().trim());
        assertEquals("<td test_id=\"last_name\">Cambria</td>", result.get(1).toString());
    }

    public void testAndCompound() throws Exception {
        HtmlRule rule = HtmlRule.onTags(TABLE);
        rule.thenAnd(AttributeRule.on("test_id", "cust_2"), AttributeRule.on("test_type", "customer"));
        rule.thenAttribute("test_id", "first_name");
        Reader reader = new StringReader(TABLE_COMPOUND_SAMPLE);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(1, result.size());
        assertEquals("Coheed", result.get(0).getText().trim());
        assertEquals("<td test_id=\"first_name\">Coheed</td>", result.get(0).toString());
    }

    public void testTextTag() throws Exception {
        HtmlRule rule = TextRule.on(".*ic.*");
        Reader reader = new StringReader(SIMPLE_TAG_SAMPLE);
        List<HtmlElement> result = new HtmlProcessor().parse(reader, rule);
        assertEquals(1, result.size());
        assertEquals("Alice", result.get(0).getText().trim());
        assertEquals("<b>Alice</b>", result.get(0).toString());
    }
}
